local Tablet = AceLibrary("Tablet-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Cartographer_Icons")
local AceOO = AceLibrary("AceOO-2.0")
L:RegisterTranslations("enUS", function() return {
	["Icon packs"] = true,
	["gather_pack"] = "Gatherer IconPack",
	["Rich Thorium"] = true,
	["Crystal"] = "Un'Goro Power Crystal",
} end)

L:RegisterTranslations("koKR", function() return {
	["Icon packs"] = "아이콘 모음",
	["gather_pack"] = "게더러용 아이콘모음",
	["Rich Thorium"] = "풍부한 토륨 광맥",
	["Crystal"] = "운고로 수정",
} end)


local scheduler
local effective_icons = {}
local overrides = {}

Cartographer_Icons = Cartographer:NewModule("Icons", "AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")
Cartographer_Icons:RegisterDB("Cartographer_IconsDB")

-- icons we have overwritten
function Cartographer_Icons:OnInitialize()
	--self.db = Cartographer_Icons:AcquireDBNamespace("Icons")
	self:RegisterDefaults("profile", {
		custom_icons = {},
	})
	-- menu options
	local aceopts = {}
	-- load our icon packs
	for k,v in pairs(self.custom) do
		if not self.db.profile.custom_icons[k] then
			self.db.profile.custom_icons[k] = {}
		end
		aceopts[k] = {
			name = L[k],
			desc = L[k],
			type = 'group',
			args = {},
			disabled = function() return not Cartographer:IsModuleActive(self) end,
		}
		for icon,texture in pairs(v) do
			if self.db.profile.custom_icons[k][icon] then
				effective_icons[icon] = k
			end
			aceopts[k].args[icon] = {
				name = L[icon],
				desc = L[icon],
				icon = texture,
				type = 'toggle',
				get = function() 
					return self.db.profile.custom_icons[k][icon]
				end,
				set = function()
					self.db.profile.custom_icons[k][icon] = not self.db.profile.custom_icons[k][icon]
					self:ApplyPack(k)
				end
			}
		end
	end
	aceopts.toggle = {
		name = AceLibrary("AceLocale-2.2"):new("Cartographer")["Enabled"],
		desc = AceLibrary("AceLocale-2.2"):new("Cartographer")["Suspend/resume this module."],
		type  = 'toggle',
		order = -1,
		get   = function() return Cartographer:IsModuleActive(self) end,
		set   = function() Cartographer:ToggleModuleActive(self) end,
	}
	Cartographer.options.args.Icons = {
		name = L["Icon packs"],
		desc = self.notes,
		type = 'group',
		args = aceopts,
		handler = self,
	}
end


function Cartographer_Icons:OnEnable()
	if not Cartographer_Notes then
		Cartographer:ToggleModuleActive(self, false)
	else
		self:OnUpdate()
		scheduler = self:ScheduleRepeatingEvent(self.OnUpdate, 1, self)	
	end
end

function Cartographer_Icons:OnUpdate()
	-- icon updater
	for k,v in pairs(effective_icons) do
		if Cartographer_Notes:IsIconRegistered(k) then
			if not overrides[k] then
				local origPath = Cartographer_Notes:OverrideIconGraphic(k,self.custom[v][k])
				overrides[k] = origPath
			else
				Cartographer_Notes:OverrideIconGraphic(k,self.custom[v][k])
			end
		end
	end
	Cartographer_Notes:RefreshMap()
end

function Cartographer_Icons:RestoreOriginal(icon)
	if overrides[icon] and Cartographer_Notes:IsIconRegistered(icon) then
		Cartographer_Notes:OverrideIconGraphic(icon,overrides[icon])
		overrides[icon] = nil
	end
end

function Cartographer_Icons:ApplyPack(pack)
	-- we need to go through all our custom packs and apply these changes
	for icon,enabled in pairs(self.db.profile.custom_icons[pack]) do
		-- is another pack hooking this icon?
		local hookedPack = effective_icons[icon]
		-- someone else if hooking lets override that hook
		if hookedPack and hookedPack ~= pack then
			-- this pack has an enablments lets remove other hook, set it to not hook and update
			if enabled then
				self.db.profile.custom_icons[hookedPack][icon] = false
				effective_icons[icon] = pack
				self:OnUpdate()
			end
		elseif hookedPack then
			if not enabled then
				-- we are already hooking this icon lets restore the original since
				effective_icons[icon] = nil
				self:RestoreOriginal(icon)
			end
		else
			if enabled then
				effective_icons[icon] = pack
				self:OnUpdate()
			end
		end
	end
end

-- works
function Cartographer_Icons:OnDisable()
	self:UnregisterAllEvents()
	self:CancelScheduledEvent(scheduler)
	-- revert the graphics back
	if Cartographer_Notes and Cartographer:IsModuleActive("Notes") then
		for k,v in pairs(effective_icons) do
			self:RestoreOriginal(k)
		end
	end
	Cartographer_Notes:RefreshMap()
end

--function Cartographer_Icons:IconUpdate()
--	-- redo the icons
--	if Cartographer_Notes then
--		local lst = Cartographer_Notes:GetIconList()
--		for k,v in pairs(self.db.profile.custom_icons) do
--			if lst[v] then
--				overrides[v] = lst[v].path
--				Cartographer_Notes:OverrideIconGraphic(v,self.custom[k])
--			end
--		end
--	end
--	Cartographer_Notes:RefreshMap()
--end

